/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *                                                                                  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *                                                                                  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/
#include "libadminutil/admutil.h"
#include <stdio.h>
#include <time.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "ldap.h"
#include "ldap_ssl.h"
#include "libadmsslutil/psetcssl.h"

#ifdef XP_WIN32
#define strcasecmp stricmp
#define strncasecmp _strnicmp
#endif

#ifndef PATH_MAX
#define PATH_MAX 256
#endif

PR_IMPLEMENT(PsetHndl)
psetRealLDAPImportSSL(PsetHndl pseth, char* ldapHost, int ldapPort,
                      int secure, char* sieDN, char* userDN, char* passwd, 
                      char* configFile, char* filter, int* errorcode)
{
  LDAP          *ld = NULL;
  PsetHndl      psethndl;
  int           ldapError, unbindF = 0;


  if ((!ldapHost) || (ldapPort < 1) || (!sieDN)) {
    /* set error code to SIEDN not available */
    *errorcode = PSET_ARGS_ERROR;
    return NULL;
  }

  /* Connect to the LDAP server  */

  if (secure) ld = ldapssl_init(ldapHost, ldapPort, 1);
  else  ld = ldap_init(ldapHost, ldapPort);

  /* authenticate to LDAP server*/
  if ((ldapError = ldap_simple_bind_s(ld, userDN, passwd))
      != LDAP_SUCCESS ) {
    switch (ldapError) {
    case LDAP_INAPPROPRIATE_AUTH:
    case LDAP_INVALID_CREDENTIALS:
    case LDAP_INSUFFICIENT_ACCESS:
      /* authenticate failed: Should not continue */
      /* ldap_perror( pset->ld, "ldap_simple_bind_s" );  */
      *errorcode = PSET_AUTH_FAIL;
      return NULL;
    case LDAP_NO_SUCH_OBJECT:
    case LDAP_ALIAS_PROBLEM:
    case LDAP_INVALID_DN_SYNTAX:
      /* Not a good DN */
      *errorcode = PSET_ENTRY_NOT_EXIST;
      return NULL;
    default:
      ldap_unbind(ld);
      unbindF = 0;
      ld = NULL;
    }
  }
  else unbindF = 1;

  psethndl = psetFullLDAPImportRef(pseth,
                                ld,
                                sieDN,
                                configFile,
                                filter,
                                unbindF,
                                errorcode, userDN, passwd);
  /*
  if (psethndl) {
    psetSetLDAPReferalInfo(psethndl, userDN, passwd);
  }
  */

  return psethndl;
}


PR_IMPLEMENT(PsetHndl)
psetRealCreateSSL(char* ldapHost, int ldapPort, int secure, char* sieDN, 
                  char* userDN, char* passwd, char* configFile, 
                  int* errorcode)
{
  return psetRealLDAPImportSSL(NULL, ldapHost, ldapPort, secure, sieDN,
                               userDN, passwd, configFile, NULL, errorcode);
}


PR_IMPLEMENT(PsetHndl)
psetCreateSSL(char* serverID, char* configRoot, char* userDN, char* passwd,
              int* errorcode)
{
  PsetHndl      pset;
  AdmldapInfo   ldapInfo= NULL, admLdapInfo=NULL;
  char          *serverRoot = getenv("NETSITE_ROOT");
  char          path[PATH_MAX], *ldapHost=NULL, *sieDN, *bindPasswd = NULL;
  char          *secureStr = NULL;
  int           ldapPort = 389, dummy, secure = 0;
  
  ldapInfo = admldapBuildInfo(configRoot, errorcode);

  if (!ldapInfo) return NULL;

  /* get LDAP info, default is localhost:389 */
  ldapHost = admldapGetHost(ldapInfo);
  ldapPort = admldapGetPort(ldapInfo);

  /* For non-admin server, if no ldap information, get ldap host and port 
     from admin server  */
  if (strcasecmp(serverID, "admin-serv")) {
    if (!ldapHost && serverRoot) {
      PR_snprintf(path, sizeof(path), "%s%cadmin-serv%cconfig",
                  serverRoot, FILE_PATHSEP, FILE_PATHSEP);
      admLdapInfo = admldapBuildInfo(configRoot, &dummy);
      ldapHost = admldapGetHost(admLdapInfo);
      ldapPort = admldapGetPort(admLdapInfo);
    }
  }

  *errorcode = PSET_OP_OK;

  if (!ldapHost) ldapHost = PL_strdup("localhost");
  secure = admldapGetSecurity(ldapInfo);
  if (secure) {
    if (ldapPort < 0) ldapPort = 636;
  }
  else {
    if (ldapPort < 0) ldapPort = 389;
  }

  /* Get SIE and password  */

  sieDN = admldapGetSIEDN(ldapInfo);
  if (!userDN) {
    userDN = sieDN;
    bindPasswd = admldapGetSIEPWD(ldapInfo);
  }
  else bindPasswd = passwd;

  if (configRoot)
    PR_snprintf(path, sizeof(path), "%s%clocal.conf", configRoot, FILE_PATHSEP);
  else
    PR_snprintf(path, sizeof(path), "%s%c%s%cconfig%clocal.conf",
                serverRoot, FILE_PATHSEP, serverID, FILE_PATHSEP, FILE_PATHSEP);

  pset = psetRealCreateSSL(ldapHost, ldapPort, secure, sieDN, userDN,
                           bindPasswd, path, errorcode);
  destroyAdmldap(ldapInfo);
  PR_Free(ldapHost);
  PR_Free(sieDN);
  if (!passwd) { if (bindPasswd) PR_Free(bindPasswd); }
  return pset;
}


